#ifndef _PA_PaletteDual
#define _PA_PaletteDual


/*! \file PA_PaletteDual.h
    \brief Everything concerning the palette system, for both screens at once

    Not much for now, only a function to load a complete palette to a given palette
*/

#include "PA_Palette.h"

#define PAL_BG PAL_BG0
#define PAL_SPRITE PAL_SPRITE0


/** @defgroup PaletteDual Palette system for Dual Screen
 *  Load palettes, change palette colors, set the gamma, etc... on both screens !
 *  @{
 */


/*! \def PA_DualLoadPal(palette, source)
    \brief
         \~english Load a 256 color palette in the Bg or Sprite palette of both screens
         \~french Charger une palette de 256 couleurs pour les fonds ou les sprites pour les 2 crans
    \param palette
         \~english Set the Bg palette or Sprite palette : PAL_BG or PAL_SPRITE
         \~french Charger pour les Bg ou les Sprites : PAL_BG ou PAL_SPRITE
    \param source
         \~english Palette name (ex : master_Palette)
         \~french Nom de la palette (ex : master_Palette)
*/
#define PA_DualLoadPal(palette, source)  {\
DMA_Copy((void*)source, (void*)palette, 256, DMA_16NOW);\
DMA_Copy((void*)(source+1024), (void*)palette, 256, DMA_16NOW);}




/*! \def PA_DualLoadPal16(palette, n_palette, source)
    \brief
         \~english Load a 16 color palette in the Bg or Sprite palette of both screens
         \~french Charger une palette de 16 couleurs pour les fonds ou les sprites pour les deux crans
    \param palette
         \~english Set the Bg palette or Obj palette : PAL_BG or PAL_SPRITE
         \~french Charger pour les Bg ou les Sprites : PAL_BG ou PAL_SPRITE
    \param n_palette
         \~english Number of the 16 color palette to load (0-15)
         \~french Numro de la palette de 16 couleurs que l'on veut charger (0-15)

    \param source
         \~english Palette name (ex : master_Palette)
         \~french Nom de la palette (ex : master_Palette)
*/
#define PA_DualLoadPal16(palette, n_palette, source) {\
DMA_Copy((void*)source, (void*)(palette + (n_palette << 5)), 16, DMA_16NOW);\
DMA_Copy((void*)source, (void*)(palette + 1024 + (n_palette << 5)), 16, DMA_16NOW);}




/*! \fn extern inline void PA_DualSetPalNeg(u32 palette)
    \brief
         \~english Set all the palette's color to negative. To undo this, simply negative again...
         \~french Ngativer une palette donne. Pour annuler, il suffit de ngativer  nouveau. 
    \param palette
         \~english Set the Bg palette or Obj palette : PAL_BG, PAL_SPRITE
         \~french Charger pour les Bg ou les Sprites : PAL_BG, PAL_SPRITE
*/
extern inline void PA_DualSetPalNeg(u32 palette) {
u16* pal = (u16*)palette;
u16 i;
	for (i = 0; i < 256; i++) {
		pal[i+512] = pal[i] = ~pal[i]; // On fout le ngatif...
	}
}




/*! \fn extern inline void PA_DualSetPal16Neg(u32 palette, u8 n_palette)
    \brief
         \~english Set 16 color palette to negative. To undo this, simply negative again...
         \~french Ngativer une palette de 16 couleurs donne. Pour annuler, il suffit de ngativer  nouveau. 
    \param palette
         \~english Set the Bg palette or Obj palette : PAL_BG, PAL_SPRITE
         \~french Charger pour les Bg ou les Sprites : PAL_BG, PAL_SPRITE
    \param n_palette
         \~english Number of the 16 color palette (0-15)
         \~french Numro de la palette de 16 couleurs (0-15)
*/
extern inline void PA_DualSetPal16Neg(u32 palette, u8 n_palette) {
u16* pal = (u16*)palette;
u16 i;
	for (i = (n_palette << 4); i < ((n_palette + 1) << 4); i++) {
		pal[i+512] = pal[i] = ~pal[i]; // On fout le ngatif...
	}
}




/*! \fn extern inline void PA_DualLoadSpriteExtPal(u8 palette_number, void* palette)
    \brief
         \~english Load a 256 color palette in the Sprite extended palettes
         \~french Charger une palette de 256 couleurs dans les palettes tendues des sprites
    \param palette_number
         \~english Palette number (0-15)
         \~french Numro de la palette (0-15)
    \param palette
         \~english Palette to load ((void*)palette_name)
         \~french Nom de la palette  charger ((void*)nom_palette)
*/
extern inline void PA_DualLoadSpriteExtPal(u8 palette_number, void* palette){
	PA_LoadSpriteExtPal(0, palette_number, palette);
	PA_LoadSpriteExtPal(1, palette_number, palette);
}




/** @} */ // end of Palette
//////////////////////////////////////////////////////////////////////





#endif


